#include"eyesquat_yoyo.hpp"
#include"screen.hpp"
#include<vector>




namespace yoyo{


enum{
  percentage_max = 100,
  percentage_min =  20,
};


namespace{


int
percentage = 100;


int
last_percentage;


using Points = std::vector<Point>;


Points
points_table[percentage_max+1];


int
inc_table[percentage_max+1];


int
base_table[percentage_max+1];


struct
Initializer
{
  Initializer();
};


Initializer::
Initializer()
{
    for(int  n = 1;  n <= percentage_max;  ++n)
    {
      inc_table[n] = percentage_max*((percentage_max*2)-n)/n;

      base_table[n] = (chunk_size/2)-((chunk_size/2)*n/percentage_max);
    }
}


Initializer
init;


void
draw_chunk(Points&  pts, const uint8_t*  src, const int  inc_v, const int  bottom, const int  x, int  y)
{
    if(x >= 0)
    {
      int  bas_v = inc_v;

        while(bas_v < bottom)
        {
            if(src[bas_v/100])
            {
              pts.emplace_back(x,y);
            }


          bas_v += inc_v;
              y +=     1;
        }
    }
}


bool
draw(Points&  pts, const ChunkBlock&  blk, const int  p, int&  x)
{
  const int  inc_v = inc_table[p];
        int  bas_v = inc_v;

  const int  base = base_table[p];

  int  tmp_x = x                      ;
               x += blk.left+blk.width;

  const int  left = (blk.left*100);

    while(bas_v < left)
    {
      bas_v += inc_v;

        if(++tmp_x >= window_w)
        {
          return true;
        }
    }


  const int   right = ((blk.left+blk.width )*100);
  const int  bottom = ((blk.top +blk.height)*100);

    while(bas_v < right)
    {
      draw_chunk(pts,blk.chunks[bas_v/100].data,inc_v,bottom,base+tmp_x,base);

        if(++tmp_x >= window_w)
        {
          return true;
        }


      bas_v += inc_v;
    }


  return false;
}


int
get_percentage()
{
  return (percentage >= percentage_min)? percentage:percentage_min;
}


}




void
reset()
{
       percentage = 100;
  last_percentage =   0;
}


void
step(int  v)
{
  static bool  magnify_flag;

    if(magnify_flag)
    {
      percentage += v;

        if(percentage >= percentage_max)
        {
            percentage = percentage_max;
          magnify_flag = false;
        }
    }

  else
    {
      percentage -= v;

        if(percentage <= 0)
        {
            percentage = 0;
          magnify_flag = true;
        }
    }
}




void
generate(const std::vector<ChunkBlock>&  blocks)
{
//  size_t  total = 0;

    for(int  i = percentage_min;  i <= percentage_max;  ++i)
    {
      auto&  pts = points_table[i];

      int  x = 0;

      pts.clear();

        for(auto&  blk: blocks)
        {
            if(draw(pts,blk,i,x))
            {
              break;
            }
        }


//      total += pts.size();

//      printf(" %3d%% - %8d(%8dkb)\n",i,pts.size(),sizeof(Point)*pts.size()/1024);
    }


//  printf("total - %8d(%8dkb)\n",total,sizeof(Point)*total/1024);
}


void
update_screen()
{
  static int  last_p;

  int  x = 0;

  int  p = get_percentage();

    if(p != last_percentage)
    {
      last_percentage = p;

      SDL_SetRenderDrawColor(renderer,0,0,0,255);

      SDL_RenderClear(renderer);

      SDL_SetRenderDrawColor(renderer,255,255,255,255);

      const auto&  pts = points_table[p];

      SDL_RenderDrawPoints(renderer,pts.data(),pts.size());
    }


  SDL_RenderPresent(renderer);
}




}




