#include"txtlin.hpp"




namespace{


SDL_Texture*     pos_texture;
SDL_Texture*  length_texture;
SDL_Texture*  screen_texture;


int  pitch;


size_t
last_pos;


void
paste_textures()
{
  SDL_Rect  dst_rect;

  dst_rect.x =                0;
  dst_rect.y = window_h-ascii_h;
  dst_rect.h =          ascii_h;

  dst_rect.w = ascii_w*8;

  SDL_RenderCopy(renderer,pos_texture,nullptr,&dst_rect);


  dst_rect.x += dst_rect.w;
  dst_rect.w  = ascii_w*9;

  SDL_RenderCopy(renderer,length_texture,nullptr,&dst_rect);
}




void
draw_chunk(uint8_t*  dst, const uint8_t*  src, int  h, int  y)
{
    while(h--)
    {
        if(*src)
        {
          dst[pitch*y] = 0xFF;
        }


      ++src;
        ++y;
    }
}


bool
draw(uint8_t*  dst, const ChunkBlock&  blk, int&  x)
{
  int  tmp_x = x                      ;
               x += blk.left+blk.width;

    rep(n,blk.width)
    {
        if(tmp_x >= window_w)
        {
          return true;
        }


        if(tmp_x >= 0)
        {
          draw_chunk(&dst[tmp_x],&blk.chunks[blk.left+n].data[blk.top],blk.height,blk.top);
        }


      ++tmp_x;
    }


  return false;
}


}


void
update_screen()
{
  SDL_SetRenderDrawColor(renderer,0,0,0,255);

  SDL_RenderClear(renderer);

  SDL_SetRenderDrawColor(renderer,255,255,255,255);

    if(get_packet_list().size())
    {
      int  x = get_head_offset();

      void*  ptr;

      SDL_LockTexture(screen_texture,nullptr,&ptr,&pitch);

      std::memset(ptr,0,(pitch*window_h));

        for(auto&  pk: get_packet_list())
        {
            if(draw((uint8_t*)ptr,*pk.block,x))
            {
              break;
            }
        }


      SDL_UnlockTexture(screen_texture);

      static const SDL_Rect  dst_rect = {0,0,window_w,window_h};

      SDL_RenderCopy(renderer,screen_texture,nullptr,&dst_rect);


      paste_textures();
    }


  SDL_RenderPresent(renderer);
}





void
ready_textures()
{
  screen_texture = SDL_CreateTexture(renderer,SDL_PIXELFORMAT_RGB332,SDL_TEXTUREACCESS_STREAMING,window_w,window_h);
     pos_texture = SDL_CreateTexture(renderer,SDL_PIXELFORMAT_RGB332,SDL_TEXTUREACCESS_STREAMING,ascii_w*8,ascii_h);
  length_texture = SDL_CreateTexture(renderer,SDL_PIXELFORMAT_RGB332,SDL_TEXTUREACCESS_STATIC   ,ascii_w*9,ascii_h);
}


void
clean_textures()
{
  SDL_DestroyTexture(screen_texture);
  SDL_DestroyTexture(   pos_texture);
  SDL_DestroyTexture(length_texture);

}




size_t
get_pos_texture_pos()
{
  return last_pos;
}


void
update_pos_texture(size_t  pos)
{
  void*  ptr;

  int  pitch;

  SDL_LockTexture(pos_texture,nullptr,&ptr,&pitch);

  std::memset(ptr,0,(pitch*ascii_h));

  drawf((uint8_t*)ptr,pitch,"%8d",pos);

  SDL_UnlockTexture(pos_texture);


  last_pos = pos;
}


void
update_length_texture()
{
  const int  w = ascii_w*9;

  std::vector<uint8_t>  buf(w*ascii_h);

  drawf(buf.data(),w,"/%8d",text::get_length());

  SDL_UpdateTexture(length_texture,nullptr,buf.data(),w);
}




