#include"txtlin_text.hpp"
#include"font.hpp"




namespace text{


namespace{


uint32_t
counts[0x10000];


std::vector<char16_t>
chars;


size_t
pos;


std::u16string
buffer;


std::u16string
load_file(const char*  path)
{
  std::u16string  s;

  auto  f = fopen(path,"rb");

    if(f)
    {
        for(;;)
        {
          auto  c = fget_utf8(f);

            if(feof(f))
            {
              break;
            }


            if(!counts[c])
            {
              chars.push_back(c);
            }


          ++counts[c];

          s += c;
        }


      fclose(f);
    }


  return std::move(s);
}


int
compare(const void*  a, const void*  b)
{
  auto  ac = *(const char16_t*)a;
  auto  bc = *(const char16_t*)b;

  auto  an = counts[ac];
  auto  bn = counts[bc];

       if(an < bn){return  1;}
  else if(an > bn){return -1;}

  return 0;
}


void
finish_reset()
{
  pos = 0;

  std::qsort(chars.data(),chars.size(),sizeof(char16_t),compare);
}


}




void
get_element(Element&  elm)
{
  elm.pos     = pos;
  elm.unicode = buffer[pos++];

    if(pos >= buffer.size())
    {
      pos = 0;
    }
}


size_t
get_length()
{
  return buffer.size();
}


size_t
reset_pos(int  new_pos)
{
    if(buffer.size())
    {
      const int  max = buffer.size();

        if(new_pos < 0)
        {
          new_pos = std::abs(max+new_pos)%max;
        }

      else
        if(new_pos >= max)
        {
          new_pos %= max;
        }


      pos = new_pos;
    }


  return pos;
}


size_t
seek_pos(int  amount)
{
  return reset_pos(pos+amount);
}





void
fill_unicodes()
{
    rep(n,0x10000)
    {
      buffer.push_back(n);
    }
}




void
clear_counts()
{
    for(auto  c: chars)
    {
      counts[c] = 0;
    }


  chars.clear();
}


void
load(const char*  path)
{
  clear_counts();

  buffer = load_file(path);

  finish_reset();
}


void
reset(const char*  txt)
{
  clear_counts();

  buffer.clear();

    for(;;)
    {
      auto  c = sget_utf8(txt);

        if(!c)
        {
          break;
        }


        if(!counts[c])
        {
          chars.push_back(c);
        }


      buffer += c;

      ++counts[c];
    }


  finish_reset();
}


void
reset(const char16_t*  txt)
{
  clear_counts();

  buffer = txt;

    for(auto  c: buffer)
    {
        if(!counts[c])
        {
          chars.push_back(c);
        }


      ++counts[c];
    }


  finish_reset();
}


const std::vector<char16_t>&
get_char_list()
{
  return chars;
}


uint32_t
get_char_count(char16_t  c)
{
  return counts[c];
}


void
print(char16_t  c)
{
  print_unicode(c);

  printf(" %8d",counts[c]);
}


}




